#include <ArduinoBLE.h>
#include "USB.h"
#include "USBHIDKeyboard.h"
USBHIDKeyboard Keyboard;


//Initializing Values:
uint8_t val1Byte = 0;
uint8_t val2Byte = 0;
uint8_t val3Byte = 0;
uint8_t val4Byte = 0;

uint16_t value1state = 0;
uint16_t value2state = 0;
uint16_t value3state = 0;
uint16_t value4state = 0;

int spacebarcheck = 0;

void setup() {
  // put your setup code here, to run once:
  Serial.begin(115200);
  Keyboard.begin(); //Begin keyboard. 
  // while (!Serial); // optionally wait for serial terminal to open
  Serial.println("MyoWare Multi Sensor Example - BLE Central");

  if (!BLE.begin()) // initialize the BLE hardware
  {
    Serial.println("starting BLE failed!");
    while (1);
  }
  Serial.println("BLE initiallized successfully");
  BLE.scanForUuid("19b10000-e8f2-537e-4f6c-d104768a1214"); // start scanning for peripheral with specified UUID
  
  Keyboard.begin();
}

void loop() {
  // Main code here, to run repeatedly:
  BLEDevice peripheral = BLE.available(); // check if a peripheral has been discovered

  if (peripheral) // discovered a peripheral, print out its info to Serial (For Bug Testing Purposes)
  {
    Serial.print("Found ");
    Serial.print(peripheral.address());
    Serial.print(" '");
    Serial.print(peripheral.localName());
    Serial.print("' ");
    Serial.print(peripheral.advertisedServiceUuid());
    Serial.println();

    if (peripheral.localName() != "MYOWARESENSOR")
    {
      return;
    }

    BLE.stopScan();

    checkUpdate(peripheral);

    Serial.println("Starting to scan for new peripherals again...");
    BLE.scanForUuid("19b10000-e8f2-537e-4f6c-d104768a1214"); // peripheral disconnected, scan again
    Serial.println("Scan has begun...");
  }
}

void checkUpdate(BLEDevice peripheral)
{
  Serial.println("Connecting ..."); // Connect to the peripheral notification

  if (peripheral.connect())
  {
    Serial.println("Connected");
  } else {
    Serial.println("Failed to connect!");
    return;
  }

  Serial.println("Discovering attributes ..."); // Discover peripheral attributes
  if (peripheral.discoverAttributes())
  {
    Serial.println("Attributes discovered");
  } else {
    Serial.println("Attribute discovery failed!");
    peripheral.disconnect();
    return;
  }

  // Retrieve the data characteristic
  BLECharacteristic dataCharacteristic = peripheral.characteristic("19b10001-e8f2-537e-4f6c-d104768a1214");

  if (!dataCharacteristic)
  {
    Serial.println("Peripheral does not have that characteristic!");
    peripheral.disconnect();
    return;
  } else if (!dataCharacteristic.canWrite())
  {
    Serial.println("Peripheral does not have a writable characteristic!");
    peripheral.disconnect();
    return;
  } else if (!dataCharacteristic.canRead())
  {
    Serial.println("Peripheral does not have a readable characteristic!");
    peripheral.disconnect();
    return;
  } else if (!dataCharacteristic.canSubscribe())
  {
    Serial.println("Characteristic is not subscribable!");
    peripheral.disconnect();
    return;
  } else if (!dataCharacteristic.subscribe())
  {
    Serial.println("subscription failed!");
    peripheral.disconnect();
    return;
  }

  //int sensorValue1 = 0;
  //analogRead(A0); // read the input on analog pin A0
  //int sensorValue2 = 0; 
  //analogRead(A3); // read the input on analog pin A3
  //int sensorValue3 = 0;
  //analogRead(A4); // read the input on analog pin A4
  //int sensorValue4 = 0;
  //analogRead(A5); // read the input on analog pin A5


  //Change Values here per Sensor:
  int delay_interval = 50;
  int delayValue = 1500;
  int delayValueUpper = 2000;

  //Sensor W/
  bool delay1 = false;
  int counter1 = 0;
  int delayCount1 = 0;
  int count1_signalLength = 500;
  int voltage_thresh_1_upper = 300;
  int voltage_thresh_1_upper_high = 600;
  int voltage_thresh_1_lower = 100;

  //Sensor A
  bool delay2 = false;
  int counter2 = 0;
  int delayCount2 = 0;
  int count2_signalLength = 500;
  int voltage_thresh_2_upper = 100;
  int voltage_thresh_2_lower = 50;

  //Sensor Spacebar
  bool delay3 = false;
  int counter3 = 0;
  int delayCount3 = 0;
  int count3_signalLength = 500;
  int voltage_thresh_3_upper_high = 300;
  int voltage_thresh_3_upper = 150;
  int voltage_thresh_3_lower = 100;

  //Sensor D
  bool delay4 = false;
  int counter4 = 0;
  int delayCount4 = 0;
  int count4_signalLength = 500;
  int voltage_thresh_4_upper = 100;
  int voltage_thresh_4_lower = 50;
  
  bool keyW = false;
  bool keyW_Alt = false;
  bool keyS = false;
  bool keyS_Space = false;
  bool keyA = false;
  bool keyD = false;

  bool keySpace = false;
  bool keyShift = false;


  while (peripheral.connected()) // while the peripheral is connected
  {
    if (dataCharacteristic.valueUpdated()) // Check to see if the value of the characteristic has been updated
    {
      uint32_t received_val = 0;
      dataCharacteristic.readValue(received_val); // note, "readValue(uint32_t& value)" needs the variable to be passed by reference

      // parse received_val - this contains all 4 of our ADC values (as each byte)
      //Serial.println(received_val, BIN); // optional print of the entire uint32_t for debugging. Suppress other prints. 
      val1Byte = (received_val & 0x000000FF);        //  read the input on analog pin A0 /left leg/ 'a' 
      val2Byte = ((received_val & 0x0000FF00) >> 8);   //  read the input on analog pin A1 /left arm/ 'w' 
      val3Byte = ((received_val & 0x00FF0000) >> 16);   //  read the input on analog pin A3 /right arm/ 'spacebar'
      val4Byte = ((received_val & 0xFF000000) >> 24);   //  read the input on analog pin A2 /right leg/ 'd' 


      uint16_t sensorValueA = map(val1Byte, 0, 255, 0, 1023); // A
      uint16_t sensorValueW = map(val2Byte, 0, 255, 0, 1023); // - W
      uint16_t sensorValueS = map(val3Byte, 0, 255, 0, 1023); // - Spacebar
      uint16_t sensorValueD = map(val4Byte, 0, 255, 0, 1023); // - D


      // W
      if (delay1 == true){
        delayCount1 += delay_interval;   
      }
      if (delayCount1 >= delayValueUpper){
          delay1 = false;
      }
      if (sensorValueW > voltage_thresh_1_upper){
        counter1 += delay_interval;
        if (counter1 >= count1_signalLength && delay1 == false){
          //Voltage Threshold Check:
          if (sensorValueW > voltage_thresh_1_upper_high){
            keyW_Alt = !keyW_Alt;
          }
          else {
            keyW = !keyW;
          }
          delay1 = true;
          delayCount1 = 0;
        }
      }
      if (sensorValueW <= voltage_thresh_1_lower){
        counter1 = 0;
      }

      // A
      if (delay2 == true){
        delayCount2 += delay_interval;   
      }
      if (delayCount2 >= delayValue){
          delay2 = false;
      }
      if (sensorValueA > voltage_thresh_2_upper){
        counter2 += delay_interval;
        if (counter2 >= count2_signalLength && delay2 == false){
          keyA = !keyA;
          delay2 = true;
          delayCount2 = 0;
        }
      }
      if (sensorValueA <= voltage_thresh_2_lower){
        counter2 = 0;
      }

      // Spacebar
      if (delay3 == true){
        delayCount3 += delay_interval;   
      }
      if (delayCount3 >= delayValueUpper){
          delay3 = false;
      }
      if (sensorValueS > voltage_thresh_3_upper){
        counter3 += delay_interval;
        if (counter3 >= count3_signalLength && delay3 == false){
          if (sensorValueS > voltage_thresh_3_upper_high){
            keyS_Space = !keyS_Space;
          }
          else {
            keyS = !keyS;
          }
           
          delay3 = true;
          delayCount3 = 0;
        }
      }
      if (sensorValueS <= voltage_thresh_3_lower){
        counter3 = 0;
      }

      // D
      if (delay4 == true){
        delayCount4 += delay_interval;   
      }
      if (delayCount4 >= delayValue){
          delay4 = false;
      }
      if (sensorValueD > voltage_thresh_4_upper){
        counter4 += delay_interval;
        if (counter4 >= count4_signalLength && delay4 == false){
          keyD = !keyD;
          delay4 = true;
          delayCount4 = 0;
        }
      }
      if (sensorValueD <= voltage_thresh_4_lower){
        counter4 = 0;
      }


      //Logic (W/Space Outputs)
      
      if (keyW == true){
        Keyboard.press('w');
        keyW_Alt = false;
      }
      else if (keyW_Alt == true){
        Keyboard.press('z');
        Keyboard.press('w');
        //keyW = false;
      }
      else if (keyW == false || keyW_Alt == false){
        Keyboard.release('w');
        Keyboard.release('z');
        keyW = false;
        keyW_Alt = false;
      }
      if (keyS == true){
        Keyboard.press(' ');
        //Keyboard.write((char) 0x20);
        delay(1);
        keyS = false;
      }
      else if (keyS_Space == true){
        Keyboard.press(' ');
      }
      else if (keyS == false || keyS_Space == false){
        Keyboard.release(' ');
        keyS = false;
        keyS_Space = false;
      }
      
      // Space
      if (keyA == true){
        Keyboard.press('a');
      }
      else if (keyA == false){
        Keyboard.release('a');
      }
      if (keyD == true){
          Keyboard.press('d');
        }
      else if (keyD == false) { 
        Keyboard.release('d');
      }

      
      // Serial Print Checks: Uncomment or Comment values you want to check for calibration/testing purposes. 
      // Serial.print(keyA);
      // Serial.print("\t");
      // Serial.print(keyD);
      // Serial.print("\t");
      // Serial.print(keyW);
      // Serial.print("\t");
      // Serial.print(keyS);
      // Serial.print("\t");
      // Serial.println(keyShift);
      
      // Serial.print(spacebarcheck);
      // Serial.print("\t");
      Serial.print(sensorValueA);
      Serial.print("\t");
      Serial.print(sensorValueW);
      Serial.print("\t");
      Serial.print(sensorValueS);
      Serial.print("\t");
      Serial.println(sensorValueD);
      delay(delay_interval);
    }
    delay(1);
  }
  Serial.println("Peripheral disconnected");
  Keyboard.releaseAll();
}