#include <ArduinoBLE.h>

BLEService sensorDataService("19b10000-e8f2-537e-4f6c-d104768a1214"); // BLE Service named "sensorDataService"
// BLE Data Characteristic - custom 128-bit UUID, readable, writable and subscribable by central
// Note, "BLENotify" is what makes it subscribable
BLEUnsignedLongCharacteristic dataCharacteristic("19b10001-e8f2-537e-4f6c-d104768a1214", BLERead | BLEWrite | BLENotify); 

void setup() 
{
  Serial.begin(115200);
  // while (!Serial); // optionally wait for serial terminal to open
  Serial.println("MyoWare_MULTIPLE_BLUETOOTH");

  if (!BLE.begin()) { // begin initialization
    Serial.println("starting BLE failed!");
    while (1);
  }
  Serial.println("BLE initiallized successfully");

  BLE.setLocalName("MYOWARESENSOR"); // set advertised local name
  BLE.setAdvertisedService(sensorDataService); // set advertised service UUID
  sensorDataService.addCharacteristic(dataCharacteristic); // add the characteristic to the service
  BLE.addService(sensorDataService); // add service
  dataCharacteristic.writeValue(0); // set the initial value for the characeristic
  BLE.advertise(); // start advertising

}

void loop() 
{  
  BLEDevice central = BLE.central(); // listen for BLE peripherals to connect

  if (central) // if a central is connected to peripheral
  {
    Serial.print("Connected to central: ");
    Serial.println(central.address()); // print the central's MAC address

    Serial.println("Reading Sensors and writing BLE characteristic values now..."); 

    while (central.connected()) 
    {
      uint16_t sensorValue1 = analogRead(A0); // read the input on analog pin A0 / left leg/ 
      uint16_t sensorValue2 = analogRead(A1); // read the input on analog pin A1 / left arm
      uint16_t sensorValue3 = analogRead(A3); // read the input on analog pin A3 /right arm/ 
      uint16_t sensorValue4 = analogRead(A2); // read the input on analog pin A2 /right leg/ 

      uint8_t Val1Byte = map(sensorValue1, 0, 1023, 0, 255);
      uint8_t Val2Byte = map(sensorValue2, 0, 1023, 0, 255);
      uint8_t Val3Byte = map(sensorValue3, 0, 1023, 0, 255);
      uint8_t Val4Byte = map(sensorValue4, 0, 1023, 0, 255);

      uint32_t output = 0; //output value 32 bit.
                                  //Sections where data is transcribed into u32bit integer to transmit
      output |= Val1Byte;         // ----  ----  ----  ----  ----  ----  XXXX  XXXX
      output |= (Val2Byte << 8);  // ----  ----  ----  ----  XXXX  XXXX  ----  ---- 
      output |= (Val3Byte << 16); // ----  ----  XXXX  XXXX  ----  ----  ----  ---- 
      output |= (Val4Byte << 24); // XXXX  XXXX  ----  ----  ----  ----  ----  ---- 

      delay(10);
      dataCharacteristic.writeValue(output);

      //Test Prints:
      //Serial.print("\t");
      //Serial.print(sensorValue1); // print out the value
      //Serial.print("\t");
      //Serial.println(sensorValue2); // print out the value
      //Serial.print("\t");
      //Serial.print(sensorValue3); // print out the value
      //Serial.print("\t");
      //Serial.println(sensorValue4); // print out the value
      
    }
    Serial.print(F("Disconnected from central: ")); // when the central disconnects, print it out
    Serial.println(central.address());
  }
}